'use strict';

const fs = require('fs-extra');
const compressing = require('compressing');
const download = require('download');
const yml = require('require-yml');
const to_yaml = require('json-to-pretty-yaml');
const Hexo = require('hexo');
const cdn = require('qcloud-cdn-node-sdk');

exports.env = (key, _default = null) => {
    return process.env[key] || _default;
};

exports.directory = function(path = '') {
    return this.env('local', false) ? __dirname + '/' + path : '/tmp/' + path;
}

exports.main_handler = (event, context, callback) => {
    const COS_SECRET_ID     = this.env('cos_secret_id');
    const COS_SECRET_KEY    = this.env('cos_secret_key');
    const COS_BUCKET        = this.env('cos_bucket');
    const COS_REGION        = this.env('cos_region');

    const CDN_OPEN          = this.env('cdn_open', false);
    const CDN_SECRET_ID     = this.env('cdn_secret_id', COS_SECRET_ID);
    const CDN_SECRET_KEY    = this.env('cdn_secret_key', COS_SECRET_KEY);
    const CDN_DOMAIN        = this.env('cdn_domain');

    const GITHUB_OWNER  = this.env('github_owner');
    const GITHUB_REPO   = this.env('github_repo');
    const GITHUB_TOKEN  = this.env('github_token');

    const BLOG_FILE_NAME = this.directory('blog.tar.gz');

    fs.emptyDirSync(this.directory());

    console.log('下载博客文件');
    download(`https://api.github.com/repos/${GITHUB_OWNER}/${GITHUB_REPO}/tarball/master`, '', {
        headers: {
            'Authorization': GITHUB_TOKEN ? `token ${GITHUB_TOKEN}` : null
        }
    }).then(data => {
        fs.writeFileSync(BLOG_FILE_NAME, data);

        console.log('解压文件');
        compressing.tgz.uncompress(BLOG_FILE_NAME, this.directory('blog'))
            .then(() => {
                const dirs = fs.readdirSync(this.directory('blog'));

                if(!dirs.length) {
                    console.log('获取目录失败');
                    return;
                }

                fs.ensureDirSync(this.directory('blog/node_modules'));
                console.log('创建node_modules文件夹');

                fs.copySync(this.directory(`blog/${dirs[0]}/`), this.directory('blog'));
                console.log('博客拷贝');

                fs.copySync(__dirname + '/node_modules', this.directory('blog/node_modules'));
                console.log('node_modules拷贝');

                const config = yml(this.directory('blog/_config.yml'));
                console.log('读取配置文件');

                config.deploy.secretId  = COS_SECRET_ID;
                config.deploy.secretKey = COS_SECRET_KEY;
                config.deploy.bucket    = COS_BUCKET;
                config.deploy.region    = COS_REGION;

                fs.writeFile(this.directory('blog/_config.yml'), to_yaml.stringify(config));

                const hexo = new Hexo(this.directory('blog'), {
                    config: this.directory('blog/_config.yml'),
                    output: this.directory('blog/'),
                });

                console.log('init');
                hexo.init().then(function() {
                    console.log('clean');
                    hexo.call('clean').then(function() {
                        console.log('generate');
                        hexo.call('generate', {
                            deploy: true
                        }).then(function() {
                            hexo.exit();

                            if(CDN_OPEN) {
                                console.log('清理CDN缓存');

                                cdn.config({
                                    secretId: CDN_SECRET_ID,
                                    secretKey: CDN_SECRET_KEY,
                                });

                                cdn.request('RefreshCdnDir', {
                                    'dirs.0': 'https://' + CDN_DOMAIN
                                }, response => {
                                    response = JSON.parse(response);
                                    console.log(response);
                                    console.log(response['code'] == 0 ? '刷新缓存成功' : response['message']);
                                });
                            }
                        });
                    });
                });
            });
    });
    callback(null, "success callback");
};
